import os
from omniidl import idlvisitor, output
import jnutils
import string

class InterfaceVisitor (idlvisitor.AstVisitor):
    def __init__(self):
        self.directoryParts = []
        self.directory = ''
        self.package = ''

    def calculateDirectory(self):
        self.directory = string.join(self.directoryParts, '/')
        self.package = string.join(self.directoryParts, '.')
        
    def visitAST(self, node):
        for declaration in node.declarations():
            declaration.accept(self)
            
    def visitModule(self, node):
        try:
            os.mkdir(node.identifier())
        except OSError:
            pass
        self.directoryParts.append(node.identifier())
        self.calculateDirectory()
        for defn in node.definitions():
            defn.accept(self)
        self.directoryParts.pop()
        self.calculateDirectory()
        
    def visitInterface(self, node):
        if not node.mainFile():
            return
        self.out = output.Stream(file(self.directory + '/' + node.identifier() + ".java", 'w'))
        self.out.out('package ' + self.directory + ';')
        self.out.out('public interface ' + node.identifier())
        inh = node.inherits()
        if len(inh) != 0:
            extends = 'extends '
            needcomma = 0
            for inherit in inh:
                if needcomma:
                    extends = extends + ", "
                else:
                    needcomma = 1
                extends = extends + jnutils.GetClassName(self.directory, inherit)
            self.out.out('  ' + extends)
        self.out.out('{')
        self.out.out('};')
        self.out = None

def run(tree):
    iv = InterfaceVisitor()
    tree.accept(iv)
